/**
 * 常用函数
 */
export default {
  clientHeight() {
    return window.innerHeight;
  },
  clientWidth() {
    return window.innerWidth;
  },
  /**
   * 函数防抖
   */
  debounce(fn, delay) {
    let timer = null;
    return function () {
      const context = this;
      const args = arguments;
      clearTimeout(timer);
      timer = setTimeout(() => {
        fn.apply(context, args);
      }, delay);
    };
  },
  /**
   * 函数节流
   */
  throttle(fn, delay) {
    let last = 0;
    return function () {
      const curr = +new Date();
      if (curr - last > delay) {
        fn.apply(this, arguments);
        last = curr;
      }
    };
  },
  /**
   * 获取鼠标绝对位置
   */
  mousePosition(ev) {
    if (ev.pageX || ev.pageY) {
      return {
        x: ev.pageX,
        y: ev.pageY,
      };
    }
    return {
      x: ev.clientX + document.documentElement.scrollLeft - document.body.clientLeft,
      y: ev.clientY + document.documentElement.scrollTop - document.body.clientTop,
    };
  },
  /**
   * 获取元素绝对位置
   */
  getElCoordinate(dom) {
    let t = dom.offsetTop;
    let l = dom.offsetLeft;
    dom = dom.offsetParent;
    while (dom) {
      t += dom.offsetTop;
      l += dom.offsetLeft;
      dom = dom.offsetParent;
    }
    return {
      top: t,
      left: l,
    };
  },
  /**
   * 数组去重
   */
  uniq(arr) {
    return Array.from(new Set(arr));
  },
  maps(arr, key, fun) {
    for (const x of arr) {
      x[key] = fun(x[key]);
    }
    return arr;
  },
  /**
   * 统一修改yyyy-MM-dd HH:mm:ss时间格式
   */
  parseDate(date) {
    return date.toLocaleString('chinese', {
      hour12: false,
    }).replace(/\//g, '-');
  },
  getValue(data, key = 'id') {
    return data.map(val => val[key]);
  },
  listToStr(data, key = 'id') {
    if (this.isArray(data)) {
      return data.map((val) => {
        if (this.isObject(val)) {
          return val[key];
        }
        return val;

      }).toString();
    }
  },
  strToList(str) {
    let arr = str.split(',');
    let arrLast = [];
    for (let i = 0; i < arr.length; i++) {
      let arrObj = arr[i].split(':');
      let obj = {};
      obj.id = arrObj[0];
      obj.name = arrObj[1];
      arrLast.push(obj);
    }
    return arrLast;
  },
  /**
   * 将children转换成id、pId格式
   */
  transformArray(arr, key) {
    const res = [];

    function build(arr) {
      for (const x of arr) {
        res.push(x);
        if (x[key]) {
          build(x[key]);
        }
      }
    }
    build(arr);
    return res;
  },
  /**
   * 重写es5 map方法
   */
  map(data, key) {
    const arr = [];
    for (const x of data) {
      arr.push(x[key]);
    }
    return arr;
  },
  /**
   * 浅拷贝
   */
  shallowCopy(src) {
    const dst = {};
    for (const prop in src) {
      if (src.hasOwnProperty(prop)) {
        dst[prop] = src[prop];
      }
    }
    return dst;
  },
  /**
   * 深拷贝
   */
  deepCopy(data) {
    let obj;
    if (this.isArray(data)) {
      obj = [];
      for (let i = 0, len = data.length; i < len; i++) {
        obj.push(this.deepCopy(data[i]));
      }
    } else if (this.isObject(data)) {
      obj = {};
      for (const key in data) {
        obj[key] = this.deepCopy(data[key]);
      }
    } else {
      return data;
    }
    return obj;
  },
  /**
   * 通过key获取数组某个对象
   */
  getKey(arr, key, value) {
    return arr.filter(val => val[key] == value)[0];
  },
  /**
   * 处理数组
   * map:{新:旧} 映射
   * add:{key:value}新增
   */
  processArray(arr, map, add) {
    arr.forEach((val) => {
      if (map) {
        for (const x in map) {
          val[x] = val[map[x]];
        }
      }
      if (add) {
        for (const x in add) {
          val[x] = add[x];
        }
      }
    });
    return arr;
  },
  /**
   * 判断字符str是否包含在obj中,obj可以是数字、字符串、数组,不区分大小写
   */
  includes(str, obj) {
    if (this.isNull(str)) {
      return false
    } else {
      if (this.isString(obj)) {
        return obj.toLowerCase().indexOf(str.toString().toLowerCase()) !== -1
      }
      if (this.isNumber(obj)) {
        return obj.toString().toLowerCase().indexOf(str.toString().toLowerCase()) !== -1
      }
      if (this.isArray(obj)) {
        return obj.some((val) => {
          return val.toString().toLowerCase().indexOf(str.toString().toLowerCase()) !== -1
        })
      } else {
        return false
      }
    }

  },
  isNull(obj) {
    return Object.prototype.toString.call(obj) == '[object Null]';
  },
  isString(obj) {
    return Object.prototype.toString.call(obj) == '[object String]';
  },
  isNumber(obj) {
    return Object.prototype.toString.call(obj) == '[object Number]';
  },
  isArray(obj) {
    return Object.prototype.toString.call(obj) == '[object Array]';
  },
  isObject(obj) {
    return Object.prototype.toString.call(obj) == '[object Object]';
  },
  isDate(obj) {
    return Object.prototype.toString.call(obj) == '[object Date]';
  },
};
